#!/usr/bin/perl

##### 'XChat-apt' script developed by the-me on irc.quakenet.org ( #Linux-Dev ) Version 0.9.0
##### This program is free software; you can redistribute it and/or modify
##### it under the terms of the GNU General Public License as published by
##### the Free Software Foundation; either version 2 of the License, or
##### (at your option) any later version.
##### This program is distributed in the hope that it will be useful,
##### but WITHOUT ANY WARRANTY; without even the implied warranty of
##### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##### GNU General Public License for more details.
##### You should have received a copy of the GNU General Public License
##### along with this program; if not, write to the Free Software
##### Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

### Xchat registers and hooks.

Xchat::register("irc.quakenet.org:6667 #Linux-Dev XChat-apt Script for XChat", "0.9.0", "This is the /apt Perlscript for XChat", "");
Xchat::print"» Loaded XChat-apt Version 0.9.0 from 'the-me' on irc.quakenet.org @ #Linux-Dev - Try /apt_help. «";
Xchat::hook_command("apt", "apt_help");
Xchat::hook_command("apt_help", "apt_help");
Xchat::hook_command("apt_list", "apt_list");
Xchat::hook_command("apt_version", "apt_version");
Xchat::hook_command("apt_switch", "apt_switch");
Xchat::hook_command("apt_search", "apt_search");
Xchat::hook_command("apt_update", "apt_update");
Xchat::hook_command("apt_install", "apt_install");
Xchat::hook_command("apt_remove", "apt_remove");
Xchat::hook_command("apt_upgrade", "apt_upgrade");
Xchat::hook_command("apt-help", "apt_help");
Xchat::hook_command("apt-list", "apt_list");
Xchat::hook_command("apt-version", "apt_version");
Xchat::hook_command("apt-search", "apt_search");
Xchat::hook_command("apt-update", "apt_update");
Xchat::hook_command("apt-install", "apt_install");
Xchat::hook_command("apt-remove", "apt_remove");
Xchat::hook_command("apt-switch", "apt_switch");
Xchat::hook_command("apt-upgrade", "apt_upgrade");

### Perlmodules.

use warnings;
use strict;
use LWP::Simple;
use IO::Socket;

### Declaration of scriptwide needed variabels.

my @servers;
my($updatesource, $xchatstarts, $automaticscriptload, $updateself, $mirror, $oldmirror);

### Important variabels, don't change it!

my $tries = 0;
my $build = '090';
my $version = '0.9.0';
my $xdir = Xchat::get_info('xchatdir');

### Start misc checks.

&apt_check_misc;

### Miscelanous checks at startup.

sub apt_check_misc () {
	if(!-e "$xdir/apt.settings"){
		my $getsets = getstore("http://89.144.5.28/repository/supplementary/apt.settings", "$xdir/apt.settings");
	}
	open(CONFIG,"<$xdir/apt.settings");
	my @config = (<CONFIG>);
	close(CONFIG);
	foreach my $temp (@config){
		chomp($temp);
		my($set, $value) = split /\s+/, $temp, 2;
		if($set eq 'updateself'){
			$updateself = $value;
		}
		elsif($set eq 'automaticscriptload'){
			$automaticscriptload = $value;
		}
		elsif($set eq 'automaticupdatesource'){
			$updatesource = $value;
		}
		elsif($set eq 'sourceupdateafterxchatstarts'){
			$xchatstarts = $value;
		}
		elsif($set eq 'addmirror'){
			push(@servers, $value);
		}
	}
	$mirror = $servers[rand@servers];
	if(&apt_mirror_check($mirror) == 0){
		&apt_mirror_new($mirror);
	}
	if(!-e "$xdir/apt.list"){
		open(CREATE, ">$xdir/apt.list");
		close(CREATE);
	}
	if(!-e "$xdir/packages.list" or !-e "$xdir/packages.source" or !-e "$xdir/packages.remove"){
		Xchat::print"..:: No packagelists are presented on this system. Getting it... ::..";
		&apt_update();
		Xchat::print"..:: This seems, that this is your first use of XChat-apt, have a look at /apt_help. ::..";
	}
	if(!-e "$xdir/apt.conf"){
		open(APTCONF, ">$xdir/apt.conf");
		print APTCONF '0';
		close(APTCONF);
	}
	open(CHECKLIST, "<$xdir/apt.list");
	my @checklist = (<CHECKLIST>);
	close(CHECKLIST);
	open(WRITECHECKLIST, ">$xdir/apt.list");
	foreach my $checktemp (@checklist){
		chomp($checktemp);
		if($checktemp){
			print WRITECHECKLIST "$checktemp\n";
		}
	}
	close(WRITECHECKLIST);
	if($updatesource == 1){
		open(REPO, "<$xdir/apt.conf");
		my $starts = int(<REPO>);
		close(REPO);
		$starts++;
		if($starts >= $xchatstarts){
			&apt_update();
			open(NEWCONF, ">$xdir/apt.conf");
			print NEWCONF '0';
			close(NEWCONF);
		} else {
			open(UPDATECONF, ">$xdir/apt.conf");
			print UPDATECONF "$starts";
			close(UPDATECONF);
		}
	}
	if($updateself == 1){
		&apt_check_update;
	}
	return 1;
}

### Config switcher for XChat-apt.

sub apt_switch {
	my $arg = $_[0][1];
	if(!$arg or $arg eq 'help'){
		Xchat::print"..:: Available options: ::..";
		Xchat::print"show      - Shows current settings.";
		Xchat::print"switch    - Changes settings. Usage: switch <option> <value>.";
		Xchat::print"status    - Shows the current status of all added mirrors.";
		Xchat::print"update    - Tries to update XChat-apt.";
		Xchat::print"rehash    - Rehashs the config files.";
		Xchat::print"list      - Lists all added mirrors.";
		Xchat::print"addmirror - Adds a mirror.";
		Xchat::print"delmirror - Deletes a mirror.";
		Xchat::print"..:: End of list. ::..";
	}
	elsif($arg eq 'update'){
		&apt_check_update;
	}
	elsif($arg eq 'rehash'){
		&apt_check_misc;
		Xchat::print"..:: Rehashed all settings. ::..";
	}
	elsif($arg eq 'show'){
		Xchat::print"..:: Current settings: ::..";
		Xchat::print"updateself                   - $updateself";
		Xchat::print"automaticscriptload          - $automaticscriptload";
		Xchat::print"automaticupdatesource        - $updatesource";
		Xchat::print"sourceupdateafterxchatstarts - $xchatstarts";
		Xchat::print"..:: End of list. ::..";
	}
	elsif($arg eq 'switch'){
		my $arg2 = $_[0][2];
		if(!$arg2 or $arg2 eq 'help'){
			Xchat::print"..:: Available options: ::..";
			Xchat::print"updateself                   - Automaticly updates XChat-apt.";
			Xchat::print"automaticscriptload          - Un/loads scripts after installing automatic.";
			Xchat::print"automaticupdatesource        - Updates automaticly after X XChat-apt starts the repository files.";
			Xchat::print"sourceupdateafterxchatstarts - The X-value for automaticupdatesource.";
			Xchat::print"Each argument only can be \"1\" or \"0\".";
			Xchat::print"The sourceupdateafterxchatstarts value can be between 1 - 1000.";
		}
		elsif($arg2 eq 'updateself'){
			my $arg3 = $_[0][1];
			if($arg3 == 1 or $arg3 == 0){
				$updateself = $arg3;
				&apt_save_conf;
				Xchat::print"..:: Changed the value of $arg2 to $arg3. ::..";
			} else {
				Xchat::print"..:: The argument $arg3 isn't valid. You can only choose between \"1\" and \"0\". ::..";
			}
		}
		elsif($arg2 eq 'automaticscriptload'){
			my $arg3 = $_[0][1];
			if($arg3 == 1 or $arg3 == 0){
				$automaticscriptload = $arg3;
				&apt_save_conf;
				Xchat::print"..:: Changed the value of $arg2 to $arg3. ::..";
			} else {
				Xchat::print"..:: The argument $arg3 isn't valid. You can only choose between \"1\" and \"0\". ::..";
			}
		}
		elsif($arg2 eq 'automaticupdatesource'){
			my $arg3 = $_[0][1];
			if($arg3 == 1 or $arg3 == 0){
				$updatesource = $arg3;
				&apt_save_conf;
				Xchat::print"..:: Changed the value of $arg2 to $arg3. ::..";
			} else {
				Xchat::print"..:: The argument $arg3 isn't valid. You can only choose between \"1\" and \"0\". ::..";
			}
		}
		elsif($arg2 eq 'sourceupdateafterxchatstarts'){
			my $arg3 = $_[0][1];
			if($arg3 >= 1 and $arg3 <= 1000){
				$xchatstarts = $arg3;
				&apt_save_conf;
				Xchat::print"..:: Changed the value of $arg2 to $arg3. ::..";
			} else {
				Xchat::print"..:: The argument $arg3 isn't valid. You can only choose between \"1\" and \"1000\". ::..";
			}
		} else {
			Xchat::print"..:: Invalid option. ::..";
		}
	}
	elsif($arg eq 'addmirror'){
		my $arg2 = $_[0][2];
		if(($arg2) and (apt_func_mirror_check("$arg2") == 0)){
			if(apt_mirror_avail("$arg2") == 0){
				push(@servers, $arg2);
				&apt_save_conf;
				Xchat::print"..:: Added the mirror $arg2. ::..";
			} else {
				Xchat::print"..:: Error. The given mirror is already added. ::..";
			}
		} else {
			if($arg2){
				Xchat::print"..:: Error. The mirror isn't reachable or valid. ::..";
			} else {
				Xchat::print"..:: Error. You have to give a mirror as argument to add. ::..";
			}
		}
	}
	elsif($arg eq 'delmirror'){
		my $arg2 = $_[0][2];
		if(!$arg2){
			Xchat::print"..:: Error. You have to give a mirror as argument to delete. ::..";
		}
		elsif(apt_mirror_avail("$arg2") == 1){
			@servers = grep{$_ ne "$arg2"} @servers;
			&apt_save_conf;
			Xchat::print"..:: I deleted the mirror $arg2. ::..";
		} else {
			Xchat::print"..:: Error. The given mirror isn't available in the mirrorlist. ::..";
		}
	}
	elsif($arg eq 'list'){
		my $counter = 0;
		Xchat::print"..:: List of all mirrors. ::..";
		foreach my $temp (@servers){
			chomp($temp);
			if($temp){
				$counter++;
				Xchat::print"$counter. $temp";
			}
		}
		Xchat::print"..:: End of list. ::..";
	}
	elsif($arg eq 'status'){
		my $counter = 0;
		my $count = scalar(@servers);
		Xchat::print"..:: Status of all $count added mirrors. ::..";
		foreach my $temp (@servers){
			chomp($temp);
			if($temp){
				$counter++;
				my $status;
				my $return = apt_func_mirror_check($temp);
				if($return == 0){
					$status = 'OK';
				}
				elsif($return == 1){
					$status = 'Invalid magic code';
				}
				elsif($return == 2){
					$status = 'Not reachable';
				}
				elsif($return == 3){
					$status = 'No repository found';
				}
				Xchat::print"$counter. $temp  -  Status: $status";
			}
		}
		Xchat::print"..:: End of list. ::..";
	} else {
		Xchat::print"..:: Invalid option. ::..";
	}
}

### Saves the new config after a switch.

sub apt_save_conf () {
	open(NEWCONFIG,">$xdir/apt.settings");
	print NEWCONFIG "updateself $updateself\nautomaticscriptload $automaticscriptload\n";
	print NEWCONFIG "automaticupdatesource $updatesource\nsourceupdateafterxchatstarts $xchatstarts\n";
	foreach my $temp (@servers){
		chomp($temp);
		if($temp){
			print NEWCONFIG "addmirror $temp\n";
		}
	}
	close(NEWCONFIG);
}

### Updateroutine für the apt-script.

sub apt_check_update () {
	if(&apt_mirror_check($mirror) == 0){
		&apt_mirror_new($mirror);
	}
	my $version = get("http://$mirror/repository/supplementary/version.info");
	if(!$version){
		Xchat::print"..:: Can't get the version infos from the mirror. Aborting! ::..";
		return 1;
	}
	elsif($version > $build){
		Xchat::print"..:: It's a new release available ($version). Now I will download and install it! ::..";
		unlink("\"$xdir/apt.pl\"");
		my $getnew = getstore("http://$mirror/repository/supplementary/apt.pl", "$xdir/apt.pl");
		if(!$getnew){
			Xchat::print"..:: Fatal error at updating to the newest XChat-apt version. Please inform a Linux-Dev administrator! ::..";
			return 1;
		}
		elsif($automaticscriptload == 1){
			Xchat::command("unload \"$xdir/apt.pl\"");
			Xchat::command("load \"$xdir/apt.pl\"");
			Xchat::print"..:: Updated and loaded the new version. ::..";
		} else {
			Xchat::print"..:: Updated to the current version. Please restart your XChat. ::..";
		}
	}
	return 1;
}

### Checkroutine for the mirror.

sub apt_mirror_check {
	my ($mirrorip) = @_;
	my $client_socket = new IO::Socket::INET (
		PeerAddr => $mirrorip,
		PeerPort => 80,
		Proto => 'tcp'
	);
	if($client_socket){
		my $magic = get("http://$mirrorip/repository/supplementary/mirror_magic");
		chomp($magic);
		if($magic eq 'h56432r8'){
			if($mirror ne $mirrorip and $mirrorip ne $oldmirror){
				Xchat::print"..:: XChat-apt uses now the following mirror: $mirrorip ::..";
			}
			$tries = 0;
		} else {
			Xchat::print"..:: Error. The mirror $mirrorip hasn't got a valid magic code! Jump to the next mirror... ::..";
			$client_socket->close();
			return 0;
		}
		$client_socket->close();
		return 1;
	}
	return 0;
}

### Checks if a mirror is already added.

sub apt_mirror_avail {
	my ($host) = @_;
	foreach my $temp (@servers){
		if($host eq $temp){
			return 1;
		}
	}
	return 0;
}

### Single function for checking mirrors.

sub apt_func_mirror_check {
	my ($mirrorip) = @_;
	my $client_socket = new IO::Socket::INET (
		PeerAddr => $mirrorip,
		PeerPort => 80,
		Proto => 'tcp'
	);
	if($client_socket){
		my $magic = get("http://$mirrorip/repository/supplementary/mirror_magic");
		if(!$magic){
			return 3;
		}
		chomp($magic);
		if($magic eq 'h56432r8'){
			$client_socket->close();
			return 0;
		}
		$client_socket->close();
		return 1;
	}
	return 2;
}

### Sets a new mirror.

sub apt_mirror_new {
	$tries++;
	$oldmirror = $_[0];
	$mirror = $servers[rand@servers];
	if(scalar(@servers) == 0){
		Xchat::print"..:: FATAL ERROR! The serverlist is empty! Aborting and unloading! ::..";
		Xchat::command("uload \"$xdir/apt.pl\"");
	}
	elsif(&apt_mirror_check($mirror) == 0 or $mirror eq $oldmirror and $tries <= 10){
		&apt_mirror_new($mirror);
	}
	elsif($tries > 10){
		Xchat::print"----------------------------------------------------------------";
		Xchat::print"..:: Too many tries to connect to a valid mirror. Aborting! ::..";
		Xchat::print"..::  Please contact me\@linux-development.eu about that!   ::..";
		Xchat::print"----------------------------------------------------------------";
	}
}

### Routine to list the packages.list

sub apt_list {
	open(LIST, "<$xdir/packages.list");
	my @packages = (<LIST>);
	close(LIST);
	open(INSTALLED, "<$xdir/apt.list");
	my @installed = (<INSTALLED>);
	close(INSTALLED);
	my $string = "XChat-apt $version";
	push(@installed, $string);
	@packages = sort {uc($a) cmp uc($b)} @packages;
	@installed = sort {uc($a) cmp uc($b)} @installed;
	Xchat::print"..:: Available packages: ::..";
	Xchat::print"..:: ------------------- ::..";
	foreach my $ptemp (@packages){
		chomp($ptemp);
		if($ptemp){
			my ($pack, $ver, $categorie) = split /\s+/ => $ptemp;
			Xchat::print"$pack - $ver - $categorie";
		}
	}
	Xchat::print"..:: ------------------- ::..";
	Xchat::print"..:: Installed packages: ::..";
	Xchat::print"..:: ------------------- ::..";
	foreach my $itemp (@installed){
		chomp($itemp);
		if($itemp){
			my ($pack, $ver) = split /\s+/ => $itemp;
			Xchat::print"$pack - $ver";
		}
	}
	Xchat::print"..:: ------------------- ::..";
	undef(@packages);
	undef(@installed);
	return 1;
}

### Routine to display current available and installed version of one package.

sub apt_version {
	my $versionpackage = $_[0][1];
	my($category, $description, $version1, $version2);
	if(!$versionpackage){
		Xchat::print"..:: You didn't enter a packagename. Have a look at /apt_help. ::..";
		return 1;
	}
	open(VERSIONLIST, "<$xdir/apt.list");
	my @versionpackages = (<VERSIONLIST>);
	close(VERSIONLIST);
	open(VERSIONAVAILABLE, "<$xdir/packages.list");
	my @availableversions = (<VERSIONAVAILABLE>);
	close(VERSIONAVAILABLE);
	foreach my $versiontemp1 (@versionpackages){
		my ($currentname, $currentversion) = split /\s+/ => $versiontemp1;
		if(uc($currentname) eq uc($versionpackage)){
			$version1 = $currentversion;
		}
	}
	foreach my $versiontemp2 (@availableversions){
		my ($availname, $availversion, $cat, $desc) = split /\s+/, $versiontemp2, 4;
		if(uc($availname) eq uc($versionpackage)){
			$version2 = $availversion;
			$category = $cat;
			$description = $desc;
		}
	}
	if(!$version1){
		$version1 = 'Not installed.';
	}
	if(!$version2){
		$version2 = 'Not available.';
	}
	if(!$category){
		$category = 'None.';
	}
	if(!$description){
		$description = 'Not available.';
	}
	Xchat::print"..:: ----------------------------------------- ::..";
	Xchat::print"..:: Versioninformations about $versionpackage ::..";
	Xchat::print"..:: ----------------------------------------- ::..";
	Xchat::print"     Package: $versionpackage";
	Xchat::print"   Categorie: $category";
	Xchat::print"   Installed: $version1";
	Xchat::print"   Available: $version2";
	Xchat::print" Description: $description";
	Xchat::print"..:: ----------------------------------------- ::..";
	return 1;
}

### Routine to search for packages.

sub apt_search {
	my $searchstring = $_[0][1];
	if(!$searchstring){
		Xchat::print"..:: You did't enter a searchstring. Have a look at /apt_help. ::..";
		return 1;
	}
	my $searchcounter = 0;
	open(SEARCHPACKAGES, "<$xdir/packages.list");
	my @searchlist = (<SEARCHPACKAGES>);
	close(SEARCHPACKAGES);
	Xchat::print"..:: ------------------------------------------ ::..";
	Xchat::print"..:: XChat-apt search results for $searchstring ::..";
	Xchat::print"..:: ------------------------------------------ ::..";
	foreach my $searchtemp (@searchlist){
		chomp($searchtemp);
		my ($spname, $spversion, $spcategorie) = split /\s+/ => $searchtemp;
		if(uc($spname) =~ uc($searchstring)){
			$searchcounter++;
			Xchat::print"..:: $spname ($spversion) - $spcategorie «";
		}
	}
	Xchat::print"..:: ------------------------------------------ ::..";
	Xchat::print"..:: I've found $searchcounter package/s matching $searchstring ::..";
	Xchat::print"..:: ------------------------------------------ ::..";
	return 1;
}

### Routine to update the sources and packages list.

sub apt_update {
	if(&apt_mirror_check($mirror) == 0){
		&apt_mirror_new($mirror);
	}
	unlink("$xdir/packages.list", "$xdir/packages.source", "$xdir/packages.remove");
	my $list = getstore("http://$mirror/repository/packages.list", "$xdir/packages.list");
	my $source = getstore("http://$mirror/repository/packages.source", "$xdir/packages.source");
	my $remove = getstore("http://$mirror/repository/packages.remove", "$xdir/packages.remove");
	if(!$list or !$source or !$remove){
		Xchat::print"..:: I can't get the newest repository. ::..";
		return 1;
	}
	Xchat::print"..:: I updated the sourcelist successful. ::..";
	return 1;
}

### Routine to install a script.

sub apt_install {
	if(&apt_mirror_check($mirror) == 0){
		&apt_mirror_new($mirror);
	}
	my $isinstalled = my $checkifavail = my $foundpackage = 0;
	my $packagename = $_[0][1];
	if(!$packagename){
		Xchat::print"..:: You didn't enter a packagename. Read please the /apt_help to know, how to use it. Aborting! ::..";
		return 1;
	}
	open(CHECKAVAIL, "<$xdir/packages.list");
	my @packageslist = (<CHECKAVAIL>);
	close(CHECKAVAIL);
	foreach my $checkifavailtemp (@packageslist){
		my ($tmpname) = split /\s+/ => $checkifavailtemp;
		if($tmpname eq $packagename){
			$checkifavail = 1;
		}
	}
	if($checkifavail == 0){
		Xchat::print"..:: Couldn't found $packagename at the packages.list. Have a look at the /apt_help. Aborting! ::..";
		return 1;
	}
	open(INSTALLED, "<$xdir/apt.list");
	my @aptlist = (<INSTALLED>);
	close(INSTALLED);
	foreach my $checkinstalled (@aptlist){
		if($checkinstalled =~ /$packagename/){
			$isinstalled = 1;
		}
	}
	if($isinstalled == 0){
		my($packageversion, $packagepack, $packagepackmorgenfile, $packagepacksource);
		foreach my $checkpackageslist (@packageslist){
			my ($checkpackname, $checkpackver) = split /\s+/ => $checkpackageslist;
			if($checkpackname eq $packagename){
				$packageversion = $checkpackver;
			}
		}
		open(SOURCELIST, "<$xdir/packages.source");
		my @sourceslist = (<SOURCELIST>);
		close(SOURCELIST);
		foreach my $checksourceslist (@sourceslist){
			my ($checksourcename, $checksourcepack) = split /\s+/ => $checksourceslist;
			if($checksourcename eq $packagename){
				$packagepack = $checksourcepack;
				$packagepack =~ s/MIRROR/$mirror/g;
			}
		}
		my @packagesfiles = get($packagepack);
		if(scalar(@packagesfiles) == 0){
			Xchat::print"..:: Can't get the list. Aborting! ::..";
			return 1;
		}
		foreach my $packagetemp (@packagesfiles){
			if($packagetemp){
				chomp($packagetemp);
				my ($packagepacksource, $packagepackfile) = split /\s+/ => $packagetemp;
				$packagepacksource =~ s/MIRROR/$mirror/g;
				my $getcheck2 = getstore("$packagepacksource", "$xdir/$packagepackfile");
				if(!$getcheck2){
					Xchat::print"..:: Can't get packagefiles. Aborting! ::..";
					return 1;
				}
				elsif($automaticscriptload == 1 and $packagepackfile  =~ /\.(?:pl|tcl|py|rb)\z/){
					Xchat::command("load \"$xdir/$packagepackfile\"");
				}
			}
		}
		push(@aptlist,"$packagename $packageversion");
		open(CNEWAPTLIST, ">$xdir/apt.list");
		foreach my $writeit (@aptlist){
			chomp($writeit);
			if($writeit){
				print CNEWAPTLIST "$writeit\n";
			}
		}
		close(CNEWAPTLIST);
		Xchat::print"..:: Installed $packagename in version $packageversion successful. ::..";
		return 1;
	}
	elsif($isinstalled == 1){
		my($installedversion, $availableversion);
		foreach my $checkinstalledversion (@aptlist){
			my ($checkname, $checkversion) = split /\s+/ => $checkinstalledversion;
			if($checkname eq $packagename){
				$installedversion = $checkversion;
			}
		}
		foreach my $availtemp (@packageslist){
			my ($availname, $availversion) = split /\s+/ => $availtemp;
			if($availname eq $packagename){
				$availableversion = $availversion;
			}
		}
		if($availableversion > $installedversion){
			open(GETSOURCESUPDATE, "<$xdir/packages.source");
			my @getsourcesupdate = (<GETSOURCESUPDATE>);
			close(GETSOURCESUPDATE);
			foreach my $sourcestemp (@getsourcesupdate){
				my ($sourcename, $sourceurl) = split /\s+/ => $sourcestemp;
				if($sourcename eq $packagename){
					$sourceurl =~ s/MIRROR/$mirror/g;
					my $getcheck3 = getstore("$sourceurl", "$xdir/$packagename");
					if(!$getcheck3){
						Xchat::print"..:: Can't get packagenames. Aborting! ::..";
						return 1;
					}
				}
			}
			open(PACKAGESOURCEPACKGET, "<$xdir/$packagename");
			my @packagesourcepackget = (<PACKAGESOURCEPACKGET>);
			close(PACKAGESOURCEPACKGET);
			foreach my $sourcefiletemp (@packagesourcepackget){
				my ($sourceurl, $sourcefilename) = split /\s+/ => $sourcefiletemp;
				$sourceurl =~ s/MIRROR/$mirror/g;
				my $getcheck4 = getstore("$sourceurl", "$xdir/$sourcefilename");
				if(!$getcheck4){
					Xchat::print"..:: Can't get sourcepackages. Aborting! ::..";
					return 1;
				}
				elsif($automaticscriptload == 1 and $sourcefilename  =~ /\.(?:pl|tcl|py|rb)\z/){
					Xchat::command("unload \"$xdir/$sourcefilename\"");
					Xchat::command("load \"$xdir/$sourcefilename\"");
				}
			}
			open(UPDATEAPTLIST, "<$xdir/apt.list");
			my @updateaptlist = (<UPDATEAPTLIST>);
			close(UPDATEAPTLIST);
			my $removestring = "$packagename $installedversion";
			@updateaptlist = grep{$_ ne "$removestring"} @updateaptlist;
			my $writenewentry = "$packagename $availableversion";
			push(@updateaptlist,$writenewentry);
			open(WRITEUPDATEDLIST, ">$xdir/apt.list");
			foreach my $uplist (@updateaptlist){
				chomp($uplist);
				if($uplist){
					print WRITEUPDATEDLIST "$uplist\n";
				}
			}
			close(WRITEUPDATEDLIST);
			Xchat::print"..:: I updated $packagename successful from version $installedversion up to $availableversion ::..";
			unlink("$xdir/$packagename");
		} else {
			Xchat::print"..:: Package $packagename is already the newest version. ::..";
		}
	}
	return 1;
}

### Routine to remove a script.

sub apt_remove {
	if(&apt_mirror_check($mirror) == 0){
		&apt_mirror_new($mirror);
	}
	my @autoremove;
	my($rem, $removed);
	my $continuerem = 0;
	my $package = $_[0][1];
	if(!$package){
		Xchat::print"..:: You didn't enter a packagename. Have a look at /apt_help. ::..";
		return 1;
	}
	open(LIST, "<$xdir/apt.list");
	my @list = (<LIST>);
	close(LIST);
	foreach my $checkcon (@list){
		if($checkcon =~ /$package/){
			$continuerem = 1;
		}
	}
	if($continuerem == 0){
		Xchat::print"..:: Can't locate package $package on your system. Aborting! ::..";
		return 1;
	} else {
		foreach my $temp (@list){
			chomp($temp);
			my ($temporarypackage, $temporaryver) = split /\s+/ => $temp;
			if($temporarypackage eq $package){
				my $remtemp = "$temporarypackage $temporaryver";
				@list = grep{$_ ne $remtemp} @list;
				open(NEWLIST, ">$xdir/apt.list");
				foreach my $newaptlist (@list){
					chomp($newaptlist);
					if($newaptlist){
						print NEWLIST "$newaptlist\n";
					}
				}
				close(NEWLIST);
				open(REMOVELIST, "<$xdir/packages.remove");
				my @removelist = (<REMOVELIST>);
				close(REMOVELIST);
				foreach my $temprem (@removelist){
					my ($remname, $remurl) = split /\s+/ => $temprem;
					if($remname eq $package){
						$rem = $remurl;
						$rem =~ s/MIRROR/$mirror/g;
					}
				}
				my @remit = get($rem);
				if(scalar(@remit) == 0){
					Xchat::print"..:: Can't get the remove package. Aborting! ::..";
					return 1;
				}
				foreach my $removefile (@remit){
					if($removefile){
						chomp($removefile);
						if($removefile  =~ /\.(?:pl|tcl|py|rb)\z/){
							Xchat::command("unload \"$xdir/$removefile\"");
						}
						my $string = "$xdir/$removefile";
						unlink($string);
					}
				}
				Xchat::print"..:: I removed $package successfully. ::..";
			}
		}
	}
	return 1;
}

### Routine to upgrade all scripts to the highest current version.

sub apt_upgrade {
	if(&apt_mirror_check($mirror) == 0){
		&apt_mirror_new($mirror);
	}
	my @autoload;
	my $upgradedpackages = 0;
	open(APTLIST, "<$xdir/apt.list");
	my @installedversion = (<APTLIST>);
	close(APTLIST);
	open(PACKAGESLIST, "<$xdir/packages.list");
	my @packageslist = (<PACKAGESLIST>);
	close(PACKAGESLIST);
	foreach my $temp1 (@installedversion){
		my ($upchecknam, $upcheckver) = split /\s+/ => $temp1;
		foreach my $temp2 (@packageslist){
			my ($availupcheckname, $availupcheckver) = split /\s+/ => $temp2;
			if($availupcheckname eq $upchecknam and $availupcheckver > $upcheckver){
				my($checkinstalledversion, $installedversion, $availableversion);
				open(GETINSTALLEDVERSION, "<$xdir/apt.list");
				my @installedversion = (<GETINSTALLEDVERSION>);
				close(GETINSTALLEDVERSION);
				foreach $checkinstalledversion (@installedversion){
					my ($insname, $insversion) = split /\s+/ => $checkinstalledversion;
					if($insname eq $upchecknam){
						$installedversion = $insversion
					}
				}
				foreach my $availtemp (@packageslist){
					my ($availname, $availversion) = split /\s+/ => $availtemp;
					if($availname eq $upchecknam){
						$availableversion = $availversion;
					}
				}
				if($availableversion > $installedversion){
					my($sourcestemp, $sourcefiletemp);
					open(GETSOURCESUPDATE, "<$xdir/packages.source");
					my @getsourcesupdate = (<GETSOURCESUPDATE>);
					close(GETSOURCESUPDATE);
					foreach $sourcestemp (@getsourcesupdate){
						my ($sourcesname, $sourcesurl) = split /\s+/ => $sourcestemp;
						if($sourcesname eq $upchecknam){
							$sourcesurl =~ s/MIRROR/$mirror/g;
							my $getcheck3 = getstore("$sourcesurl", "$xdir/$upchecknam");
							if(!$getcheck3){
								Xchat::print"..:: Can't get packagenames. Aborting! ::..";
								return 1;
							}
						}
					}
					open(PACKAGESOURCEPACKGET, "<$xdir/$upchecknam");
					my @packagesourcepackget = (<PACKAGESOURCEPACKGET>);
					close(PACKAGESOURCEPACKGET);
					foreach $sourcefiletemp (@packagesourcepackget){
						my ($sourceurl, $sourcefilename) = split /\s+/ => $sourcefiletemp;
						$sourceurl =~ s/MIRROR/$mirror/g;
						my $getcheck4 = getstore("$sourceurl", "$xdir/$sourcefilename");
						if(!$getcheck4){
							Xchat::print"..:: Can't get sourcepackages. Aborting! ::..";
							return 1;
						}
						elsif($automaticscriptload == 1 and $sourcefilename  =~ /\.(?:pl|tcl|py|rb)\z/){
							push(@autoload, $sourcefilename);
						}
					}
					open(UPDATEAPTLIST, "<$xdir/apt.list");
					my @updateaptlist = (<UPDATEAPTLIST>);
					close(UPDATEAPTLIST);
					my $removestring = "$upchecknam $installedversion";
					chomp(@updateaptlist);
					@updateaptlist = grep{$_ ne $removestring} @updateaptlist;
					my $writenewentry = "$upchecknam $availableversion";
					push(@updateaptlist,$writenewentry);
					open(WRITEUPDATEDLIST, ">$xdir/apt.list");
					foreach my $applist (@updateaptlist){
						chomp($applist);
						if($applist){
							print WRITEUPDATEDLIST "$applist\n";
						}
					}
					close(WRITEUPDATEDLIST);
					unlink("$xdir/$upchecknam");
					$upgradedpackages++;
				}
			}
		}
	}
	if($automaticscriptload == 1 and scalar(@autoload) != 0){
		foreach my $temp (@autoload){
			Xchat::command("unload \"$xdir/$temp\"");
			Xchat::command("load \"$xdir/$temp\"");
		}
		undef(@autoload);
	}
	if($upgradedpackages == 0){
		Xchat::print"..:: No upgrades available. ::..";
	} else {
		Xchat::print"..:: I upgraded $upgradedpackages packages. ::..";
	}
	return 1;
}

### Helproutine for XChat-apt.

sub apt_help {
	Xchat::print <<END;
..:: --------------- Help for XChat-apt --------------- ::..
------------------------------------------------------------
/apt - This will list this helpfile.
/apt_help - This does the same as /apt.
/apt_switch - Config switcher and checktools.
/apt_list - This will list all available packages and installed packages.
/apt_version - This displays the current installed and available version of a package and his description. Use: /apt_version <packagename>.
/apt_search - You can search packages with /apt_search <searchword>.
/apt_install - Installs a package to your XChat from our repository. Use: /apt_install <packagename>.
/apt_remove - This removes a package which you has installed with XChat-apt. Use: /apt_remove <packagename>.
/apt_upgrade - This upgrades all your installed scripts to the highest available version.
/apt_update - This will update the packagefiles.
------------------------------------------------------------
..:: Note: You can use all command with - instead of _. For example: /apt-version. ::..
END
}

#####
##### That's all, folks!
#####